@@ -1,4 +1,4 @@
-package Audit::Log 0.002;
+package Audit::Log 0.003;
 
 use strict;
 use warnings;
@@ -21,6 +21,7 @@ sub search {
     my $ret      = [];
     my $in_block = 1;
     my $line     = -1;
+    my ( $cwd, $exe, $comm ) = ( '', '', '' );
     open( my $fh, '<', $self->{path} );
   LINE: while (<$fh>) {
         next if index( $_, 'SYSCALL' ) < 0 && !$in_block;
@@ -28,10 +29,19 @@ sub search {
         # I am trying to cheat here to snag the timestamp.
         my $msg_start = index( $_, 'msg=audit(' ) + 10;
         my $msg_end   = index( $_, ':' );
-        my $timestamp = substr( $_, $msg_start, $msg_end - $msg_start ) . "\n";
+        my $timestamp = substr( $_, $msg_start, $msg_end - $msg_start );
         next if $options{older} && $timestamp > $options{older};
         next if $options{newer} && $timestamp < $options{newer};
 
+        # Snag CWDs
+        if ( index( $_, 'type=CWD' ) == 0 ) {
+            my $cwd_start = index( $_, 'cwd="' ) + 5;
+            my $cwd_end   = index( $_, "\n" ) - 1;
+            $cwd = substr( $_, $cwd_start, $cwd_end - $cwd_start );
+            $line++;
+            next;
+        }
+
         # Replace GROUP SEPARATOR usage with simple spaces
         s/[\x1D]/ /g;
 
@@ -46,12 +56,17 @@ sub search {
         } split( / /, $_ );
 
         $line++;
-        $parsed{line} = $line;
-        chomp $timestamp;
+        $parsed{line}      = $line;
         $parsed{timestamp} = $timestamp;
+        $parsed{cwd}       = $cwd;
+        $parsed{exe}  //= $exe;
+        $parsed{comm} //= $comm;
 
         if ( exists $options{key} && $parsed{type} eq 'SYSCALL' ) {
             $in_block = $parsed{key} =~ $options{key};
+            $exe      = $parsed{exe};
+            $comm     = $parsed{comm};
+            $cwd      = '';
             next unless $in_block;
         }
 
@@ -88,14 +103,15 @@ Audit::Log - auditd log parser with no external dependencies, using no perl feat
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
     my $parser = Audit::Log->new();
     my $rows = $parser->search(
         type     => qr/path/i,
-        nametype => qr/delete|create/i,
+        nametype => qr/delete|create|normal/i,
+        name     => qr/somefile.txt/i,
     );
 
 =head1 WHY
@@ -119,6 +135,17 @@ if none is provided.
 
 Also can filter returned keys by the provided array to not allocate unnecesarily in low mem situations.
 
+=head3 using with ausearch
+
+It's common to have the audit log be quite verbose, and log-rotated.
+To get around that you can dump pieces of the audit log as appropriate with ausearch.
+Here's an example of dumping keyed events for the last day, which you could then load into new().
+
+    ausearch --raw --key backupwatch -ts `date --date yesterday '+%x'` > yesterdays-audit.log
+
+Even then the audit log is quite likely to only have a few days of retention.
+Be sure to stash results appropriately.
+
 =head1 METHODS
 
 =head2 search(key => constraint) = ARRAY[HashRef{}]
@@ -128,7 +155,7 @@ If no constraints are provided, all matching rows will be returned.
 
 Example:
 
-    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create/i );
+    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create|normal/i );
 
 The above effectively will get you a list of all file modifications/creations/deletions in watched directories.
 
@@ -142,7 +169,7 @@ We can speed up processing by ignoring events of the incorrect key.
 
 Example:
 
-    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create/i, key => qr/backup_watch/i );
+    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create|normal/i, key => qr/backup_watch/i );
 
 The above will ignore events from all rules save those from the "backup_watch" rule.
 
@@ -154,9 +181,25 @@ Pass in 'older' and 'newer', and we can filter out things appropriately.
 Example:
 
     # Get all records that are from the last 24 hours
-    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create/i, newer => ( time - 86400 ) );
+    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create|normal/i, newer => ( time - 86400 ) );
 
-Handling rotated logs is left as an exercise for the reader.
+=head3 Getting full paths with CWDs
+
+PATH records don't actually store the full path to what is acted upon unless the process acting upon it used an absolute path.
+Thankfully, SYSCALL records are are always followed by a CWD record.  As such we add the 'cwd' field to all subsequent records.
+As such, you can build full paths like so:
+
+    my $parser = Audit::Log->new(undef, 'name', 'cwd');
+    my $rows = $parser->search( type => qr/path/i, nametype=qr/delete|create|normal/i );
+    my @full_paths = map { "$_->{cwd}/$_->{name}" } @$rows;
+
+=head3 Filtering by command
+
+SYSCALL records store the command which executed the call.  This is exposed as part of the parse for each child record, such as PATH or DAEMON_* records.
+Example of getting all the commands run which triggered audit events:
+
+    my $parser = Audit::Log->new(undef, 'exe')
+    my $rows = $parser->search();
 
 =head1 BUGS
 
